body {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	text-decoration: none;
	font-weight: normal;
	color: #000000;
}

a {
	text-decoration: none
}

a:link {
	color: #0000ff;
}

a:hover {
	text-decoration: underline;
}

a:visited {
	color: #880000;
}

h1 {
	font-size: 16pt;
	font-weight: normal;
	color: #62328c;
}

h2 {
	font-size: 14pt;
	font-weight: normal;
	color: #62328c;
}

h3 {
	font-size: 12pt;
	font-weight: normal;
	color: #62328c;
}

h4 {
	font-size: 11pt;
	font-weight: normal;
	color: #62328c;
}

body {
	font-size: 10pt;
}

p {
	font-size: 10pt;
}

th {
	font-size: 10pt;
	font-weight: bold;
}

td {
	font-size: 10pt;
}

.modifiable {
	background-color: #9acd32;
}

.idle {
	background-color: silver;
}

.active {
	background-color: red;
}
